"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Base = exports.Api = exports.Axios = void 0;
const commands_1 = require("./commands");
const isString = (val) => typeof val === 'string' || ((!!val && typeof val === 'object') && Object.prototype.toString.call(val) === '[object String]');
const isObject = (val) => typeof val === 'object';
const axios_1 = require("axios");
const rxjs_1 = require("rxjs");
const R = require("ramda");
const log = R.bind(console.log, console);
class Axios {
    constructor() { }
    get(url, config) { return (0, rxjs_1.defer)(async () => await axios_1.default.get(url, config)); }
    post(url, data, config) { return (0, rxjs_1.defer)(async () => await axios_1.default.post(url, data, config)); }
}
exports.Axios = Axios;
class Api {
    constructor(host, http) {
        this.host = 'https://t6.bitfire.io/';
        this.http = (http) ? http : new Axios();
        if (host)
            this.host = host;
        Api.API = this;
        log(this);
    }
    get(endpoint, config) {
        log(this.host + endpoint);
        return this.http.get(this.host + endpoint, config);
    }
    post(endpoint, data, config) {
        // log(this.http);
        return this.http.post(this.host + endpoint, data, config);
    }
}
exports.Api = Api;
class Base {
    constructor(auth, url = 'http://localhost:2890/') {
        this.auth = auth;
        this.url = url;
        this.commands = commands_1.commands;
        this.undefinedFunctions = Object.keys(this.commands).map(method => {
            if (this[method])
                return;
            const mycommands = this.commands[method];
            this[method] = (args) => this.call(method.toLowerCase(), args);
            return this.getFunctionString(method, mycommands);
        })
            .filter(item => item !== undefined)
            .sort((a, b) => a.localeCompare(b));
    }
    parseParams(method, args) {
        const commandParams = this.commands[method];
        return commandParams.map(arg => {
            if (isString(arg))
                return args[arg];
            if (isObject(arg)) {
                const key = Object.keys(arg)[0];
                const value = args[key];
                return (typeof value !== 'undefined') ? value : arg[key]; // default
            }
        });
    }
    getFunctionString(method, params) {
        const defaultToStr = (d) => isString(d) ? `'${d}'` : (Array.isArray(d) ? '[]' : (isObject(d) ? '{}' : d));
        const getDefaultStr = (param) => (param[Object.keys(param)[0]] === undefined) ? '?' : ' = ' + defaultToStr(param[Object.keys(param)[0]]);
        const fixStr = (param, addDefault = false) => param.toString().replace(/-/g, '_');
        const toStr = (param, addDefault = false) => (typeof param === 'string') ?
            fixStr(param) : fixStr(Object.keys(param)[0]) + (addDefault ? getDefaultStr(param) : '');
        const toParams = (str, v, i, arr) => str + toStr(v, true) + (i === (arr.length - 1) ? '' : ', '); // TODO: add default
        const toArray = (str, v, i, arr) => str + toStr(v) + (i === (arr.length - 1) ? '' : ', ');
        return method + '(' + params.reduce(toParams, '') + '): Observable<any> { return this.call(\'' + method.toLowerCase()
            + '\', [' + params.reduce(toArray, '') + ']); }';
    }
    call(method, args) {
        // console.log('getIfno::: ', arguments.callee.toString() );
        const params = args ? Array.isArray(args) ? args : this.parseParams(method, args) : [];
        const payload = { method: method, params: params, id: 1, jsonrpc: '2.0' };
        return (0, rxjs_1.defer)(async () => {
            var _a, _b, _c;
            let response;
            try {
                response = await axios_1.default.post(this.url, payload, { auth: this.auth });
            }
            catch (err) {
                let body = (_a = err.response) === null || _a === void 0 ? void 0 : _a.data;
                try {
                    body = JSON.stringify(body);
                }
                catch (err) { }
                throw new Error(`${method} ${(_b = err.response) === null || _b === void 0 ? void 0 : _b.status} ${(_c = err.response) === null || _c === void 0 ? void 0 : _c.statusText} ` + body);
            }
            if (response.status !== 200) {
                let body = method;
                try {
                    body = method + JSON.stringify(response.data.error);
                }
                catch (err) { }
                throw new Error(`${response.status} ${response.statusText} ` + body);
            }
            if (!response.data) {
                console.log(response);
                throw new Error(method + ' No body for: ' + this.url);
            }
            return response.data.result;
        });
    }
}
exports.Base = Base;
