"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cmd = exports.logHttpError = exports.dump = exports.log = void 0;
const commander_1 = require("commander");
const R = require("ramda");
const index_1 = require("../index");
exports.log = R.bind(console.log, console);
const ini = require("node-ini");
const dump = (obj) => console.dir(obj, { depth: null });
exports.dump = dump;
const logHttpError = (e) => {
    if (e.response)
        (0, exports.log)(e.response.data);
    else
        (0, exports.log)(e);
};
exports.logHttpError = logHttpError;
// import { Cmd } from './cmd';
// export class NewCmd extends Cmd {
// 	options = [
// 		['-v, --verbose', 'Verbose'],
// 		['-p, --production', 'Production Mode'],
// 	];
// 	commands = {
// 		sendTest: async () => {
// 		},
// 	};
// 	async init() {}
// }
// new NewCmd().run();
class Cmd {
    constructor() {
        this.options = [];
        this.log = exports.log;
        this.exitOnComplete = true;
    }
    run() {
        const { version } = require('../../package.json');
        this.program = new commander_1.Command();
        this.program = this.program.version(version);
        this.options.forEach((o) => this.program.option(o[0], o[1]));
        // .option('-v, --verbose', 'Verbose')
        // .option('-p, --production', 'Production Mode')
        this.program.argument('command')
            .argument('[args...]')
            .action((cmd, args) => {
            this.options = this.program.opts();
            (0, exports.log)('command start');
            (0, exports.log)(this.options);
            // dump(this.program);
            // args.pop();
            // args.pop();
            (0, exports.log)(cmd, args);
            this.launchCommand(cmd, args)
                .then().catch(err => { (0, exports.log)(err); });
        })
            .parse(process.argv);
    }
    async init() {
        const homedir = process.env.HOME || process.env.HOMEPATH || process.env.USERPROFILE;
        const cfg1 = ini.parseSync(homedir + '/.bitcoin/bitcoin.conf');
        const url = `http://localhost:${cfg1.rpcport}/`;
        (0, exports.log)('bchain path: ', homedir + '/.bitcoin/bitcoin.conf');
        (0, exports.log)('bchain url: ', url);
        (0, exports.log)('bchain conf: ', cfg1);
        this.bchain = new index_1.BChain({ username: cfg1.rpcuser, password: cfg1.rpcpassword }, url);
    }
    async launchCommand(cmd, args) {
        await this.init();
        cmd = this.commands[cmd];
        if (!cmd) {
            (0, exports.log)('command options:', R.keys(this.commands));
            return;
        }
        await cmd(...args);
        (0, exports.log)('this.exit', this.exitOnComplete);
        if (this.exitOnComplete)
            process.exit();
    }
}
exports.Cmd = Cmd;
